/*
 * AIEngine a new generation network intrusion detection system.
 *
 * Copyright (C) 2013-2017  Luis Campo Giralte
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301, USA.
 *
 * Written by Luis Campo Giralte <luis.camp0.2009@gmail.com> 
 *
 */
#include "test_dns.h"

#define BOOST_TEST_DYN_LINK
#ifdef STAND_ALONE
#define BOOST_TEST_MODULE dnstest
#endif
#include <boost/test/unit_test.hpp>

using namespace aiengine;

BOOST_FIXTURE_TEST_SUITE(dns_test_suite,StackDNStest)

BOOST_AUTO_TEST_CASE (test01)
{
        unsigned char *pkt = reinterpret_cast <unsigned char*> (raw_packet_ethernet_ip_udp_dns_as_dot_com);
        int length = raw_packet_ethernet_ip_udp_dns_as_dot_com_length;
        Packet packet(pkt,length);

	inject(packet);

        // Check the results
        BOOST_CHECK(ip->getTotalPackets() == 1);
        BOOST_CHECK(ip->getTotalValidatedPackets() == 1);
        BOOST_CHECK(ip->getTotalBytes() == 56);
        BOOST_CHECK(ip->getTotalMalformedPackets() == 0);

        // dns 
        BOOST_CHECK(dns->getTotalPackets() == 1);
        BOOST_CHECK(dns->getTotalBytes() == 28);
        BOOST_CHECK(dns->getTotalMalformedPackets() == 0);
	BOOST_CHECK(dns->getTotalAllowQueries() == 1);
	BOOST_CHECK(dns->getTotalBanQueries() == 0);

	Flow *flow = udp->getCurrentFlow();

	BOOST_CHECK( flow != nullptr);
        BOOST_CHECK(flow->layer7info != nullptr);
        SharedPointer<DNSInfo> dom = flow->getDNSInfo();

	BOOST_CHECK( dom->getQueryType() == static_cast<uint16_t>(DNSQueryTypes::DNS_TYPE_A));	

	std::string domain("www.as.com");

	BOOST_CHECK(domain.compare(dom->name->getName()) == 0);

	CounterMap c = dns->getCounters();
}

// Test the ban functionality for avoid unwanted domains
BOOST_AUTO_TEST_CASE (test02)
{
        unsigned char *pkt = reinterpret_cast <unsigned char*> (raw_packet_ethernet_ip_udp_dns_as_dot_com);
        int length = raw_packet_ethernet_ip_udp_dns_as_dot_com_length;
        Packet packet(pkt,length,0);

	auto host_ban_mng = SharedPointer<DomainNameManager>(new DomainNameManager());
	auto host_name = SharedPointer<DomainName>(new DomainName("unwanted domain",".com"));

	dns->setDomainNameBanManager(host_ban_mng);
	host_ban_mng->addDomainName(host_name);

	inject(packet);

        // Check the results
        BOOST_CHECK(ip->getTotalPackets() == 1);
        BOOST_CHECK(ip->getTotalValidatedPackets() == 1);
        BOOST_CHECK(ip->getTotalBytes() == 56);
        BOOST_CHECK(ip->getTotalMalformedPackets() == 0);

        // dns
        BOOST_CHECK(dns->getTotalPackets() == 1);
        BOOST_CHECK(dns->getTotalBytes() == 28);
        BOOST_CHECK(dns->getTotalMalformedPackets() == 0);
	BOOST_CHECK( dns->getTotalAllowQueries() == 0);
	BOOST_CHECK( dns->getTotalBanQueries() == 1);

	Flow *flow = udp->getCurrentFlow();

	BOOST_CHECK( flow != nullptr);
        BOOST_CHECK(flow->layer7info != nullptr);
	SharedPointer<DNSInfo> info = flow->getDNSInfo();
        BOOST_CHECK(info->name == nullptr);
        BOOST_CHECK(info->getIsBanned() == true);
}

BOOST_AUTO_TEST_CASE (test03)
{
        unsigned char *pkt = reinterpret_cast <unsigned char*> (raw_packet_ethernet_ip_udp_dns_query_srv);
        int length = raw_packet_ethernet_ip_udp_dns_query_srv_length;
        Packet packet(pkt,length);

	inject(packet);

        Flow *flow = udp->getCurrentFlow();

        BOOST_CHECK( flow != nullptr);
        BOOST_CHECK(flow->layer7info != nullptr);
        SharedPointer<DNSInfo> dom = flow->getDNSInfo();
        BOOST_CHECK( dom->getQueryType() == static_cast<uint16_t>(DNSQueryTypes::DNS_TYPE_SRV));
}

BOOST_AUTO_TEST_CASE (test04)
{
        unsigned char *pkt = reinterpret_cast <unsigned char*> (raw_packet_ethernet_ip_udp_dns_query_soa);
        int length = raw_packet_ethernet_ip_udp_dns_query_soa_length;
        Packet packet(pkt,length);

	inject(packet);

        Flow *flow = udp->getCurrentFlow();

        BOOST_CHECK( flow != nullptr);
        BOOST_CHECK(flow->layer7info != nullptr);
        SharedPointer<DNSInfo> dom = flow->getDNSInfo();
        BOOST_CHECK( dom->getQueryType() == static_cast<uint16_t>(DNSQueryTypes::DNS_TYPE_SOA));
}

BOOST_AUTO_TEST_CASE (test05)
{
        unsigned char *pkt = reinterpret_cast <unsigned char*> (raw_packet_ethernet_ip_udp_dns_dynamic_update_soa);
        int length = raw_packet_ethernet_ip_udp_dns_dynamic_update_soa_length;
        Packet packet(pkt,length);

	inject(packet);

        Flow *flow = udp->getCurrentFlow();

        BOOST_CHECK( flow != nullptr);
        BOOST_CHECK(flow->layer7info != nullptr);
        SharedPointer<DNSInfo> dom = flow->getDNSInfo();
        BOOST_CHECK( dom->getQueryType() == static_cast<uint16_t>(DNSQueryTypes::DNS_TYPE_SOA));

	std::string domain("bgskrot.ex");
	BOOST_CHECK(domain.compare(dom->name->getName()) == 0);
}

BOOST_AUTO_TEST_CASE (test06)
{
        unsigned char *pkt = reinterpret_cast <unsigned char*> (raw_packet_ethernet_ip_udp_dns_query_aaaa);
        int length = raw_packet_ethernet_ip_udp_dns_query_aaaa_length;
        Packet packet(pkt,length);

	inject(packet);

        Flow *flow = udp->getCurrentFlow();

        BOOST_CHECK( flow != nullptr);
        BOOST_CHECK(flow->layer7info != nullptr);
        SharedPointer<DNSInfo> dom = flow->getDNSInfo();
        BOOST_CHECK( dom->getQueryType() == static_cast<uint16_t>(DNSQueryTypes::DNS_TYPE_AAAA));

        std::string domain("ssl.google-analytics.com");
        BOOST_CHECK(domain.compare(dom->name->getName()) == 0);
}

BOOST_AUTO_TEST_CASE (test07)
{
        unsigned char *pkt = reinterpret_cast <unsigned char*> (raw_packet_ethernet_ip_udp_dns_dnskey_root);
        int length = raw_packet_ethernet_ip_udp_dns_dnskey_root_length;
        Packet packet(pkt,length);

	inject(packet);

        Flow *flow = udp->getCurrentFlow();

	BOOST_CHECK( flow != nullptr);
        BOOST_CHECK(flow->layer7info != nullptr);
        SharedPointer<DNSInfo> dom = flow->getDNSInfo();
        BOOST_CHECK( dom->getQueryType() == static_cast<uint16_t>(DNSQueryTypes::DNS_TYPE_DNSKEY));

        std::string domain("<Root>");
        BOOST_CHECK(domain.compare(dom->name->getName()) == 0);
}

BOOST_AUTO_TEST_CASE (test08)
{
        unsigned char *pkt = reinterpret_cast <unsigned char*> (raw_packet_ethernet_ip_udp_dns_dnskey_ietfdotorg);
        int length = raw_packet_ethernet_ip_udp_dns_dnskey_ietfdotorg_length;
        Packet packet(pkt,length);

	inject(packet);

        Flow *flow = udp->getCurrentFlow();

        BOOST_CHECK( flow != nullptr);
        BOOST_CHECK(flow->layer7info != nullptr);
        SharedPointer<DNSInfo> dom = flow->getDNSInfo();
        BOOST_CHECK( dom->getQueryType() == static_cast<uint16_t>(DNSQueryTypes::DNS_TYPE_DNSKEY));

        std::string domain("ietf.org");
        BOOST_CHECK(domain.compare(dom->name->getName()) == 0);
}

BOOST_AUTO_TEST_CASE (test09)
{
        unsigned char *pkt = reinterpret_cast <unsigned char*> (raw_packet_ethernet_ip_udp_dns_dnskey_ietfdotorg);
        int length = raw_packet_ethernet_ip_udp_dns_dnskey_ietfdotorg_length;
        Packet packet(pkt,length);

	inject(packet);

        Flow *flow = udp->getCurrentFlow();

        BOOST_CHECK( flow != nullptr);
        BOOST_CHECK( flow->layer7info != nullptr);

	//show();
	dns->releaseCache();
	//show();
        BOOST_CHECK( flow->layer7info == nullptr);
	BOOST_CHECK( flow->getDNSInfo() == nullptr);
}

// Process query and response
BOOST_AUTO_TEST_CASE (test10)
{
        unsigned char *pkt1 = reinterpret_cast <unsigned char*> (raw_packet_ethernet_ip_udp_dns_query_youtube);
        int length1 = raw_packet_ethernet_ip_udp_dns_query_youtube_length;
        Packet packet1(pkt1,length1);
        
	unsigned char *pkt2 = reinterpret_cast <unsigned char*> (raw_packet_ethernet_ip_udp_dns_response_youtube);
        int length2 = raw_packet_ethernet_ip_udp_dns_response_youtube_length;
        Packet packet2(pkt2,length2);

	inject(packet1);
	inject(packet2);

        Flow *flow = udp->getCurrentFlow();
	//show();
        BOOST_CHECK( flow != nullptr);
        BOOST_CHECK( flow->layer7info != nullptr);
        SharedPointer<DNSInfo> dom = flow->getDNSInfo();
	int i = 0;
	for (auto &ip: *dom) {
		++i;
	} 
	BOOST_CHECK( i == 0);// There is no DomainNameManager so the IPs are not extracted
}

// Process query and response and IP address extraction
BOOST_AUTO_TEST_CASE (test11)
{
        unsigned char *pkt1 = reinterpret_cast <unsigned char*> (raw_packet_ethernet_ip_udp_dns_query_youtube);
        int length1 = raw_packet_ethernet_ip_udp_dns_query_youtube_length;
        Packet packet1(pkt1,length1);

        unsigned char *pkt2 = reinterpret_cast <unsigned char*> (raw_packet_ethernet_ip_udp_dns_response_youtube);
        int length2 = raw_packet_ethernet_ip_udp_dns_response_youtube_length;
        Packet packet2(pkt2,length2);

        auto dom_mng = SharedPointer<DomainNameManager>(new DomainNameManager());
        auto dom_name = SharedPointer<DomainName>(new DomainName("Youtube test",".youtube.com"));

        dns->setDomainNameManager(dom_mng);
        dom_mng->addDomainName(dom_name);

	inject(packet1);
	inject(packet2);

	BOOST_CHECK(dns->getTotalEvents() == 1);

        Flow *flow = udp->getCurrentFlow();
        //show();
        BOOST_CHECK( flow != nullptr);
        BOOST_CHECK( flow->layer7info != nullptr);
        SharedPointer<DNSInfo> dom = flow->getDNSInfo();

	std::set<std::string> ips {
		{ "74.125.24.139" },	
		{ "74.125.24.138" },	
		{ "74.125.24.100" },	
		{ "74.125.24.101" },	
		{ "74.125.24.102" },	
		{ "74.125.24.113" },
                { "video-stats.l.google" }
	};
        int i = 0;
	std::set<std::string>::iterator it = ips.end();

        for (auto &ip: *dom) {
               	BOOST_CHECK( ips.find(ip) != it);
		++i; 
        }
        BOOST_CHECK( i == 7);
}

BOOST_AUTO_TEST_CASE (test12)
{
        unsigned char *pkt1 = reinterpret_cast <unsigned char*> (raw_packet_ethernet_ip_udp_dns_query_youtube);
        int length1 = raw_packet_ethernet_ip_udp_dns_query_youtube_length;
        Packet packet1(pkt1,length1);

        unsigned char *pkt2 = reinterpret_cast <unsigned char*> (raw_packet_ethernet_ip_udp_dns_response_youtube);
        int length2 = raw_packet_ethernet_ip_udp_dns_response_youtube_length;
        Packet packet2(pkt2,length2);

	inject(packet1);

        Flow *flow = udp->getCurrentFlow();
	BOOST_CHECK( flow != nullptr);
	BOOST_CHECK( flow->getDNSInfo() != nullptr);

	dns->releaseCache();

	BOOST_CHECK( flow != nullptr);
	BOOST_CHECK( flow->getDNSInfo() == nullptr);

	inject(packet2);
	
	BOOST_CHECK( flow != nullptr);
	BOOST_CHECK( flow->getDNSInfo() != nullptr);
}

BOOST_AUTO_TEST_CASE (test13)
{
        unsigned char *pkt1 = reinterpret_cast <unsigned char*> (raw_packet_ethernet_ip_udp_dns_random_query_google);
        int length1 = raw_packet_ethernet_ip_udp_dns_random_query_google_length;
        Packet packet1(pkt1,length1);

        unsigned char *pkt2 = reinterpret_cast <unsigned char*> (raw_packet_ethernet_ip_udp_dns_random_response_google);
        int length2 = raw_packet_ethernet_ip_udp_dns_random_response_google_length;
        Packet packet2(pkt2,length2);

        SharedPointer<DomainNameManager> dm = SharedPointer<DomainNameManager>(new DomainNameManager());
        SharedPointer<DomainName> dom_name = SharedPointer<DomainName>(new DomainName("test",".google.com"));

        dns->setDomainNameManager(dm);
        dm->addDomainName(dom_name);

        inject(packet1);
        inject(packet2);

        Flow *flow = udp->getCurrentFlow();
        BOOST_CHECK( flow != nullptr);
        BOOST_CHECK( flow->layer7info != nullptr);
        SharedPointer<DNSInfo> dom = flow->getDNSInfo();

        std::string domain("ds95g6opkyh8s8ldq1xqrucrnlzbdrii8o6r1zvggtssutwdvtyh5an27ujrkt4.99b9vbsqmrsa9jjylcoglpeemtm3uahoomlisrs6oz4gntdlul1te.99b9vbsqm");
        BOOST_CHECK(domain.compare(dom->name->getName()) == 0);

	BOOST_CHECK(dns->getTotalEvents() == 1);
	BOOST_CHECK(flow->getPacketAnomaly() == PacketAnomalyType::DNS_LONG_NAME);

}

BOOST_AUTO_TEST_CASE (test14)
{
        unsigned char *pkt1 = reinterpret_cast <unsigned char*> (raw_packet_ethernet_ip_udp_dns_query_adobe);
        int length1 = raw_packet_ethernet_ip_udp_dns_query_adobe_length;
        Packet packet1(pkt1,length1);

        unsigned char *pkt2 = reinterpret_cast <unsigned char*> (raw_packet_ethernet_ip_udp_dns_response_adobe);
        int length2 = raw_packet_ethernet_ip_udp_dns_response_adobe_length;
        Packet packet2(pkt2,length2);

        SharedPointer<DomainNameManager> dm = SharedPointer<DomainNameManager>(new DomainNameManager());
        SharedPointer<DomainName> dom_name = SharedPointer<DomainName>(new DomainName("test",".adobe.com"));

        dns->setDomainNameManager(dm);
        dm->addDomainName(dom_name);

        inject(packet1);
        inject(packet2);

        Flow *flow = udp->getCurrentFlow();
        BOOST_CHECK( flow != nullptr);
        BOOST_CHECK( flow->layer7info != nullptr);
        SharedPointer<DNSInfo> dom = flow->getDNSInfo();

	BOOST_CHECK(dns->getTotalEvents() == 1);

        std::set<std::string> items {
                { "wwwimages.wip4" },
                { "wwwimages.adobe.com.edgesuite.net" },
                { "wwwimages.adobe.com.edgesuite.net.globalredir.akadns" },
                { "a1953.x.akamai" },
                { "150.199.100.104" },
                { "150.199.100.101" }
        };
        int i = 0;
        std::set<std::string>::iterator it = items.end();

        for (auto &ip: *dom) {
                BOOST_CHECK( items.find(ip) != it);
                ++i;
        }
        BOOST_CHECK( i == 6);
}

BOOST_AUTO_TEST_CASE (test15)
{
        unsigned char *pkt = reinterpret_cast <unsigned char*> (raw_packet_ethernet_ip_udp_dns_query_ixfr);
        int length = raw_packet_ethernet_ip_udp_dns_query_ixfr_length;
        Packet packet(pkt,length);

        inject(packet);

        Flow *flow = udp->getCurrentFlow();
        BOOST_CHECK( flow != nullptr);
        BOOST_CHECK( flow->layer7info != nullptr);
        SharedPointer<DNSInfo> dom = flow->getDNSInfo();

        BOOST_CHECK( dom->getQueryType() == static_cast<uint16_t>(DNSQueryTypes::DNS_TYPE_IXFR));
        std::string domain("etas.com");
        BOOST_CHECK(domain.compare(dom->name->getName()) == 0);
}

/* Just used for check the speed of push_back against emplace_back
BOOST_AUTO_TEST_CASE (test15)
{
        SharedPointer<DNSInfo> dom = SharedPointer<DNSInfo>(new DNSInfo());

        std::vector<std::string> items {
                { "wwwimages.wip4" },
                { "wwwimages.adobe.com.edgesuite.net" },
                { "wwwimages.adobe.com.edgesuite.net.globalredir.akadns" },
                { "a1953.x.akamai" },
                { "150.199.100.104" },
                { "150.199.100.101" }
        };

	for (int j = 0 ; j < 2000000; ++j ) {
	 	for (int i = 0; i < 6; ++i) {
			dom->addIPAddress(items[i].c_str());
		}
	}
}
*/

BOOST_AUTO_TEST_SUITE_END( )

