/*
 * AIEngine a new generation network intrusion detection system.
 *
 * Copyright (C) 2013-2017  Luis Campo Giralte
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301, USA.
 *
 * Written by Luis Campo Giralte <luis.camp0.2009@gmail.com> 
 *
 */
#pragma GCC diagnostic ignored "-Wwrite-strings"
#ifndef SRC_PROTOCOLS_UDP_UDPPROTOCOL_H_
#define SRC_PROTOCOLS_UDP_UDPPROTOCOL_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "Multiplexer.h"
#include "Protocol.h"
#include <netinet/udp.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include "flow/FlowManager.h"
#include "flow/FlowCache.h"
#include "FlowForwarder.h"
#include "DatabaseAdaptor.h"

namespace aiengine {

class UDPProtocol: public Protocol {
public:
    	explicit UDPProtocol(const std::string& name,const std::string& short_name);
	explicit UDPProtocol():UDPProtocol(UDPProtocol::default_name,UDPProtocol::default_short_name) {}
    	virtual ~UDPProtocol() {}

        static constexpr char *default_name = "UDPProtocol";
        static constexpr char *default_short_name = "udp";
	static const uint16_t id = IPPROTO_UDP;
	static const int header_size = 8;

	int getHeaderSize() const { return header_size; }

	void processFlow(Flow *flow) {} // This protocol generates flows but not for destination.
	bool processPacket(Packet& packet);

	void statistics(std::basic_ostream<char>& out) { statistics(out, stats_level_); }
	void statistics(std::basic_ostream<char>& out, int level);
	void statistics() { statistics(std::cout); }

        void releaseCache() {} // No need to free cache

        void setHeader(unsigned char *raw_packet) {
        
                udp_header_ = reinterpret_cast <struct udphdr*> (raw_packet);
        }

	// Condition for say that a packet is UDP 
	bool udpChecker(Packet &packet); 
	
#if defined(__FREEBSD__) || defined(__OPENBSD__) || defined(__DARWIN__)
	uint16_t getSourcePort() const { return ntohs(udp_header_->uh_sport); }
    	uint16_t getDestinationPort() const { return ntohs(udp_header_->uh_dport); }
    	uint16_t getLength() const { return ntohs(udp_header_->uh_ulen); }
    	unsigned int getPayloadLength() const { return ntohs(udp_header_->uh_ulen) - sizeof(struct udphdr); }
#else
	uint16_t getSourcePort() const { return ntohs(udp_header_->source); }
    	uint16_t getDestinationPort() const { return ntohs(udp_header_->dest); }
    	uint16_t getLength() const { return ntohs(udp_header_->len); }
    	unsigned int getPayloadLength() const { return ntohs(udp_header_->len) - sizeof(udphdr); }
#endif
    	unsigned int getHeaderLength() const { return sizeof(struct udphdr); }
	unsigned char* getPayload() const { return (unsigned char*)udp_header_ +getHeaderLength(); }

	void setFlowManager(FlowManagerPtr flow_mng) { flow_table_ = flow_mng;}
	FlowManagerPtr getFlowManager() { return flow_table_; }
	void setFlowCache(FlowCachePtr flow_cache) { flow_cache_ = flow_cache;}
	FlowCachePtr getFlowCache() { return flow_cache_;}

	void addRejectFunction(std::function <void (Flow*)> reject) { reject_func_ = reject; }

	void setRegexManager(const SharedPointer<RegexManager>& rm) { rm_ = rm;}

	Flow *getCurrentFlow() { return current_flow_;} // used just for testing pourposes

        void increaseAllocatedMemory(int value);
        void decreaseAllocatedMemory(int value);

	int64_t getCurrentUseMemory() const; 
	int64_t getAllocatedMemory() const;
	int64_t getTotalAllocatedMemory() const;
	int64_t getAllocatedMemory(int value) const; 

        void setDynamicAllocatedMemory(bool value); 
        bool isDynamicAllocatedMemory() const;

	int32_t getTotalCacheMisses() const;
	int32_t getTotalEvents() const { return total_events_; }

	CounterMap getCounters() const; 

	void setAnomalyManager(SharedPointer<AnomalyManager> amng) { anomaly_ = amng; }

private:
	SharedPointer<Flow> getFlow(const Packet& packet); 
	void default_reject_function(Flow *flow) {}

	FlowManagerPtr flow_table_;
	FlowCachePtr flow_cache_;
	SharedPointer<RegexManager> rm_;
	struct udphdr *udp_header_;
	Flow *current_flow_;
	int32_t total_events_;
	time_t last_timeout_;
	time_t packet_time_;

	std::function <void (Flow*)> reject_func_;
	SharedPointer<AnomalyManager> anomaly_;
};

typedef std::shared_ptr<UDPProtocol> UDPProtocolPtr;

} // namespace aiengine

#endif  // SRC_PROTOCOLS_UDP_UDPPROTOCOL_H_
